<?php

declare(strict_types=1);

namespace Drupal\joinup_group\Form;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Access\AccessManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Drupal\joinup_group\Entity\GroupInterface;
use Drupal\joinup_group\Traits\MembershipCreateTrait;
use Drupal\joinup_workflow\ArchivableEntityInterface;
use Drupal\og\MembershipManagerInterface;
use Drupal\og\OgMembershipInterface;
use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * A simple form with a button to join or leave a collection.
 */
abstract class JoinGroupFormBase extends FormBase implements JoinGroupFormInterface {

  use MembershipCreateTrait;

  /**
   * The group to join.
   *
   * @var \Drupal\joinup_group\Entity\GroupInterface
   */
  protected GroupInterface $group;

  /**
   * The current user.
   *
   * @var \Drupal\user\UserInterface
   */
  protected UserInterface $user;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'join_group_form';
  }

  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
    protected MembershipManagerInterface $membershipManager,
    protected AccessManagerInterface $accessManager,
    protected $messenger,
    protected FormBuilderInterface $formBuilder,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('og.membership_manager'),
      $container->get('access_manager'),
      $container->get('messenger'),
      $container->get('form_builder')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getJoinSubmitLabel(): TranslatableMarkup {
    return $this->t('Join this :type', [
      ':type' => $this->group->bundle(),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getLeaveSubmitLabel(): TranslatableMarkup {
    return $this->t('Leave this :type', [
      ':type' => $this->group->bundle(),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getGroup(): GroupInterface {
    return $this->group;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, ?AccountProxyInterface $user = NULL, ?GroupInterface $group = NULL): array {
    assert($group instanceof GroupInterface, 'Only groups can be joined.');

    $form['#access'] = $this->access();
    $this->group = $group;
    $this->user = $this->loadUser((int) $user->id());

    $membership = $this->user->isAnonymous() ? NULL : $this->getUserNonBlockedMembership();
    $parameters = ['node' => $this->group->id()];

    // User should not be allowed to subscribe if entity it archived.
    if (empty($membership) && $group instanceof ArchivableEntityInterface && $group->isArchived()) {
      $form['#access'] = FALSE;
    }

    // Inform anonymous users that they need to authenticate to join the group.
    elseif ($this->user->isAnonymous()) {
      if ($this->accessManager->checkNamedRoute('joinup_group.authenticate_to_join', $parameters)) {
        $form['authenticate'] = [
          '#type' => 'link',
          '#title' => $this->getJoinSubmitLabel(),
          '#url' => Url::fromRoute('joinup_group.authenticate_to_join', $parameters),
          '#attributes' => [
            'class' => ['use-ajax'],
            'data-dialog-type' => 'modal',
            'data-dialog-options' => Json::encode([
              'width' => 'auto',
            ]),
          ],
        ];
        $form['#attached']['library'][] = 'core/drupal.ajax';
      }
    }

    // In case the user is not a member or does not have a pending membership,
    // give the possibility to request one.
    elseif (empty($membership)) {
      $form['join'] = [
        '#type' => 'submit',
        '#value' => $this->getJoinSubmitLabel(),
        '#name' => 'join',
      ];
    }

    // If the user is already a member of the group, show a link to the
    // membership cancellation confirmation form, disguised as a form submit
    // button. The confirmation form should open in a modal dialog for
    // JavaScript-enabled browsers.
    elseif ($membership->getState() === OgMembershipInterface::STATE_ACTIVE) {
      $form['leave'] = [
        '#theme' => 'group_leave_button',
      ];
      if ($this->accessManager->checkNamedRoute('joinup_group.leave_confirm_form', $parameters)) {
        $form['leave']['#confirm'] = [
          '#type' => 'link',
          '#title' => $this->getLeaveSubmitLabel(),
          '#url' => Url::fromRoute('joinup_group.leave_confirm_form', $parameters),
          '#attributes' => [
            'class' => ['use-ajax'],
            'data-dialog-type' => 'modal',
            'data-dialog-options' => Json::encode(['width' => 'auto']),
          ],
        ];
      }
      $form['#attached']['library'][] = 'core/drupal.ajax';
    }

    // If the user has a pending membership, do not allow to request a new one.
    elseif ($membership->getState() === OgMembershipInterface::STATE_PENDING) {
      $form['pending'] = [
        '#type' => 'link',
        '#title' => $this->t('Membership is pending'),
        '#url' => Url::fromRoute('<current>', [], ['fragment' => 'pending']),
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    parent::validateForm($form, $form_state);

    // Only authenticated users can join a group.
    if ($this->user->isAnonymous()) {
      $form_state->setErrorByName('user', $this->t('<a href=":login">Sign in</a> or <a href=":register">register</a> to change your group membership.', [
        ':login' => Url::fromRoute('user.login')->toString(),
        ':register' => Url::fromRoute('user.register')->toString(),
      ]));
    }

    $membership = $this->getUserNonBlockedMembership();
    // Make sure the user is not already a member.
    if (!empty($membership)) {
      $form_state->setErrorByName('group', $this->t('You already are a member of this group.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $membership = $this->createSaveNewMembership(OgMembershipInterface::STATE_ACTIVE, $this->user, $this->group);

    $this->messenger->addStatus($this->group->getNewMembershipSuccessMessage($membership));
  }

  /**
   * Access check for the form.
   *
   * @return bool
   *   True if the form can be accessed, false otherwise.
   */
  public function access(): bool {
    return $this->getRouteMatch()->getRouteName() !== 'joinup_group.leave_confirm_form';
  }

  /**
   * Returns a membership of the user that is active or pending.
   *
   * @return \Drupal\og\OgMembershipInterface|null
   *   The membership of the user or null.
   */
  protected function getUserNonBlockedMembership(): ?OgMembershipInterface {
    return $this->membershipManager->getMembership($this->group, $this->user->id(), [
      OgMembershipInterface::STATE_ACTIVE,
      OgMembershipInterface::STATE_PENDING,
    ]);
  }

  /**
   * Loads the user with the given ID.
   *
   * @param int $user_id
   *   The user ID.
   *
   * @return \Drupal\user\UserInterface
   *   The user entity.
   */
  protected function loadUser(int $user_id): UserInterface {
    return $this->entityTypeManager->getStorage('user')->load($user_id);
  }

}
