<?php

declare(strict_types=1);

namespace Drupal\joinup_group\Form;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\RedirectCommand;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\joinup_group\Entity\GroupInterface;
use Drupal\node\NodeInterface;

/**
 * Form to unshare a community content from within groups.
 */
class UnshareForm extends ShareFormBase {

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, ?NodeInterface $node = NULL): array {
    return $this->doBuildForm($form, $form_state, $node);
  }

  /**
   * Access callback for the unshare form.
   *
   * @param \Drupal\node\NodeInterface $node
   *   The node entity.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function access(NodeInterface $node): AccessResultInterface {
    $this->entity = $node;

    return AccessResult::allowedIf(!empty($this->getGroups()));
  }

  /**
   * {@inheritdoc}
   */
  public function getTitle(EntityInterface $node) {
    return $this->t('Unshare %title from', ['%title' => $node->label()]);
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'unshare_content_form';
  }

  /**
   * Form builder for the unshare form.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param \Drupal\Core\Entity\FieldableEntityInterface|null $entity
   *   The entity being unshared.
   *
   * @return array
   *   The form structure.
   */
  public function doBuildForm(array $form, FormStateInterface $form_state, ?FieldableEntityInterface $entity = NULL): array {
    $this->entity = $entity;

    $options = array_map(
      fn (GroupInterface $group) => $group->label(),
      $this->getGroups()
    );

    $form['groups'] = [
      '#type' => 'checkboxes',
      '#title' => 'Groups',
      '#options' => $options,
      '#default_value' => [],
    ];

    $form['actions'] = ['#type' => 'actions'];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
    ];

    if ($this->isModal() || $this->isAjaxForm()) {
      $form['actions']['submit']['#ajax'] = [
        'callback' => '::ajaxSubmit',
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $groups = [];
    // We can safely loop through these ids, as invalid options are handled
    // already by Drupal.
    foreach ($form_state->getValue('groups') as $id => $checked) {
      if ($checked) {
        $group = $this->entityTypeManager->getStorage('node')->load($id);
        assert($group instanceof GroupInterface);
        $this->removeFromGroup($group);
        $groups[] = $group->label();
      }
    }

    if (!empty($groups)) {
      $this->messenger()->addStatus('Item was unshared from the following groups: ' . implode(', ', $groups) . '.');
    }
    $form_state->setRedirectUrl($this->entity->toUrl());
  }

  /**
   * Ajax callback to close the modal.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   An ajax response that will close the modal.
   *
   * @throws \Drupal\Core\Entity\EntityMalformedException
   *   Thrown when the canonical URL cannot be generated for the shared entity.
   */
  public function ajaxSubmit(array &$form, FormStateInterface $form_state): AjaxResponse {
    $response = new AjaxResponse();
    $response->addCommand(new RedirectCommand((string) $this->getRequest()->query->get('destination')));

    return $response;
  }

  /**
   * Gets a list of groups where the content can be unshared from the user.
   *
   * @return \Drupal\joinup_group\Entity\GroupInterface[]
   *   A list of groups where the user is a facilitator and the content is
   *   shared.
   */
  protected function getGroups(): array {
    $groups = $this->getAlreadySharedGroupIds();
    if (empty($groups)) {
      return $groups;
    }

    if ($this->currentUser->hasPermission('administer shared entities')) {
      return $this->entityTypeManager->getStorage('node')->loadMultiple($groups);
    }
    return array_intersect_key($this->getUserGroupsByPermission($this->getPermissionForAction('unshare')), array_flip($groups));
  }

  /**
   * Removes the current node from being shared inside a group.
   *
   * @param \Drupal\joinup_group\Entity\GroupInterface $group
   *   The group where to remove the node.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   *   Thrown when the unshared entity cannot be saved after the group is
   *   removed from it.
   * @throws \Drupal\Core\TypedData\Exception\ReadOnlyException
   *   Thrown when the entity storage is read only.
   */
  protected function removeFromGroup(GroupInterface $group): void {
    // Flipping is needed to easily unset the value.
    $current_ids = array_flip($this->getAlreadySharedGroupIds());
    unset($current_ids[$group->id()]);
    $this->entity->get(self::FIELD_NAME_SHARED_IN)->setValue(array_flip($current_ids));
    $this->entity->skip_notification = TRUE;
    $this->entity->save();
    $this->invalidator->invalidateTags($group->getCacheTagsToInvalidate());
  }

}
